//classes: CSimpleCell, CData, CResults 
//---------------------------------------------------------------------------

#ifndef SimpleCellH
#define SimpleCellH
//---------------------------------------------------------------------------


#define nMTypes 14
// indexes for molecules
#define iATG    0   // G ATP-actines
#define iADG    1   // G ADP-actines
#define iATF    2   // F ATP-actines
#define iADF    3   // F ADP-actines

#define iFTB    4   // filament barbed ends, ATP-actin on top
#define iFDB    5   // filament barbed ends, ADP-actin on top
#define iFTP    6   // filament pointed ends, ATP-actin on top
#define iFDP    7   // filament pointed ends, ADP-actin on top

#define iCBG    8   // barbed-end capping protein (free)
#define iCBF    9   // barbed-end capping protein (bound)

#define iCPG    10  // pointed-end capping protein (free)
#define iCPF    11  // pointed-end capping protein (bound)

#define iFOG    12  // formin (free)
#define iFOF    13  // formin (bound)


#define nRTypes 21
// indexes for reactions
#define iSNUC   0   // Filament spontaneous nucleation
#define iFNUC   1   // Formin nucleation
#define iCBNU   2   // Barbed-end capper nucleation
#define iCPNU   3   // Pointed-end capper nucleation

#define iASTB   4   // Association ATP-actin at barbed end
#define iASDB   5   // Association ADP-actin at barbed end
#define iASTP   6   // Association ATP-actin at pointed end
#define iASDP   7   // Association ADP-actin at pointed end

#define iDITB   8   // Dissociation ATP-actin at barbed end
#define iDIDB   9   // Dissociation ADP-actin at barbed end
#define iDITP   10  // Dissociation ATP-actin at pointed
#define iDIDP   11  // Dissociation ATP-actin at pointed

#define iTTOD   12  // Change from ATF TO ADF in filaments
#define iDTOT   13  // Change from ADF TO ATF in solution

#define iASCB   14  // Capping of barbed end
#define iASCP   15  // Capping of pointed end
#define iDICB   16  // Uncapping of barbed end
#define iDICP   17  // Uncapping of pointed end

#define iASFB   18  // Association of formin to barbed end
#define iDIFB   19  // Dissociation of formin from barbed end
#define iFASB   20  // Formin-initiated barbed-end association


#define epsilon 1e-30  // Very small number (considered as zero)
#define epsilon2 2e-30
#define infin 1e+25  // Very big number (considered as infinity)
#define size_step 10000 // Increment/Decrement size step for dynamic arrays

//typedef double real;
class CSimpleCell {
 public:

  double v; //volume
  double t; //time

  // numbers of molecules
  int N[nMTypes];
  double C[nMTypes];

  // rate constans
  double K[nRTypes];
  // concentration-dependent rate constants
  double A[nRTypes];
  double A_old[nRTypes]; //for Gibson-Bruck method
  //pointers on arrays with A values
  double *pA;
  double *pA_old;

 	// reaction times
  double T[nRTypes];
  double t1[nRTypes];   //for Gibson-Bruck method
  int NR[nRTypes];

  int dis_count,dis_actin;
  int dis_count0,dis_actin0;

  int run_cnt;
  double run_time;

  int nf,nfmax;

  double meanlength;
  bool KillShortFilament;

  int itmin;    // index of the minimal time

  CSimpleCell(void);
  virtual ~CSimpleCell(void);


  // simulate reactions via First Reaction method
  int SimulateFR(double tmax);

  // simulate reactions via Direct method Gillespie
  int SimulateDG(double tmax);

  // simulate reactions via Gibson-Bruck method
  int SimulateGB(double tmax);

  // calculate mean filament length
  virtual int GetMinLength(void)=0;
  virtual int GetMaxLength(void)=0;
  virtual void GetLengthDistribution(int *B, int *F, int vmin, int vmax, int nbins)=0;

  // Return number of filaments with l=3
  virtual int GetLength3(void)=0;

  //Return length of filament with index nm
  virtual int RetFilLen(int nm)=0;

  // add filament to the system
  virtual int AddFilament(int nactin)=0;
  // delete filament from the system
  virtual int DelFilament(int ifil)=0;


  // Simulate dissociation of the filament #ifil
  virtual int SimDissociate(int ifil)=0;

  // Simulate stochastic filament formation
  virtual int SimSNUC(void)=0;
  // Simulate formin filament formation
  virtual int SimFNUC(void)=0;
  // Simulate barbed capping protein nuctleation
  virtual int SimCBNU(void)=0;
  // Simulate pointed capping protein nuctleation
  virtual int SimCPNU(void)=0;


  // Simulate ATP-actin association at barbed end
  virtual int SimASTB(void)=0;
  // Simulate ADP-actin association at barbed end
  virtual int SimASDB(void)=0;
  // Simulate ATP-actin association at pointed end
  virtual int SimASTP(void)=0;
  // Simulate ADP-actin association at pointed end
  virtual int SimASDP(void)=0;


  // Simulate dissociation at ATP-barbed end
  virtual int SimDITB(void)=0;
  // Simulate dissociation at ADP-barbed end
  virtual int SimDIDB(void)=0;
  // Simulate dissociation at ATP-pointed end
  virtual int SimDITP(void)=0;
  // Simulate dissociation at ADP-pointed end
  virtual int SimDIDP(void)=0;

  // Simulate aging of F-actins
  virtual int SimTTOD(void)=0;
  // Simulate recharge of G-actins
  virtual int SimDTOT(void)=0;

  // Capping of barbed end
  virtual int SimASCB(void)=0;
  // Uncapping of barbed end
  virtual int SimDICB(void)=0;
  // Capping of pointed end
  virtual int SimASCP(void)=0;
  // Uncapping of pointed end
  virtual int SimDICP(void)=0;

  // Association of formin to barbed end
  virtual int SimASFB(void)=0;
  // Dissociation of formin from barbed end
  virtual int SimDIFB(void)=0;
  // Formin-initiated barbed-end association
  virtual int SimFASB(void)=0;
};

class CData {
 public:
  int n;
  double *TIME;// time original
  double *T; // time corrected
  double *DATA;// data original
  double *D; // data corrected
  double min,max, timeshift, background, amplitude;
  CData(int _n);
  ~CData();
  void SetParam(double _tsh,double _bgr, double _amp);
  void Rescale(bool _CalcMinMax);
  double GetMin(void);
  double GetMax(void);
};

class CResults {
 public:
  int nt;
  int nd;
  double *T;
  double **DATA;
  CResults(int _nd, int _nt);
  ~CResults();
};


#endif
